@@ -15,7 +15,7 @@ __title__ = 'discord'
 __author__ = 'Rapptz'
 __license__ = 'MIT'
 __copyright__ = 'Copyright 2015-2020 Rapptz'
-__version__ = '1.3.4'
+__version__ = '1.4.0'
 
 from collections import namedtuple
 import logging
@@ -28,7 +28,7 @@ from .partial_emoji import PartialEmoji
 from .activity import *
 from .channel import *
 from .guild import Guild
-from .flags import SystemChannelFlags, MessageFlags
+from .flags import *
 from .relationship import Relationship
 from .member import Member, VoiceState
 from .message import Message, Attachment
@@ -39,14 +39,17 @@ from .permissions import Permissions, PermissionOverwrite
 from .role import Role
 from .file import File
 from .colour import Color, Colour
+from .integrations import Integration, IntegrationAccount
 from .invite import Invite, PartialInviteChannel, PartialInviteGuild
+from .template import Template
 from .widget import Widget, WidgetMember, WidgetChannel
 from .object import Object
 from .reaction import Reaction
 from . import utils, opus, abc
 from .enums import *
 from .embeds import Embed
-from .shard import AutoShardedClient
+from .mentions import AllowedMentions
+from .shard import AutoShardedClient, ShardInfo
 from .player import *
 from .webhook import *
 from .voice_client import VoiceClient
@@ -56,7 +59,7 @@ from .team import *
 
 VersionInfo = namedtuple('VersionInfo', 'major minor micro releaselevel serial')
 
-version_info = VersionInfo(major=1, minor=3, micro=4, releaselevel='final', serial=0)
+version_info = VersionInfo(major=1, minor=4, micro=0, releaselevel='final', serial=0)
 
 try:
     from logging import NullHandler
