@@ -50,6 +50,8 @@ __all__ = (
     'TeamMembershipState',
     'Theme',
     'WebhookType',
+    'ExpireBehaviour',
+    'ExpireBehavior'
 )
 
 def _create_value_cls(name):
@@ -221,6 +223,7 @@ class VerificationLevel(Enum):
     table_flip        = 3
     extreme           = 4
     double_table_flip = 4
+    very_high         = 4
 
     def __str__(self):
         return self.name
@@ -393,12 +396,18 @@ class UserFlags(Enum):
     partner = 2
     hypesquad = 4
     bug_hunter = 8
+    mfa_sms = 16
+    premium_promo_dismissed = 32
     hypesquad_bravery = 64
     hypesquad_brilliance = 128
     hypesquad_balance = 256
     early_supporter = 512
     team_user = 1024
     system = 4096
+    has_unread_urgent_messages = 8192
+    bug_hunter_level_2 = 16384
+    verified_bot = 65536
+    verified_bot_developer = 131072
 
 class ActivityType(Enum):
     unknown = -1
@@ -428,6 +437,12 @@ class WebhookType(Enum):
     incoming = 1
     channel_follower = 2
 
+class ExpireBehaviour(Enum):
+    remove_role = 0
+    kick = 1
+
+ExpireBehavior = ExpireBehaviour
+
 def try_enum(cls, val):
     """A function that tries to turn the value into enum ``cls``.
 
