@@ -25,7 +25,7 @@ DEALINGS IN THE SOFTWARE.
 """
 
 import asyncio
-from collections import namedtuple
+from collections import namedtuple, deque
 import concurrent.futures
 import json
 import logging
@@ -33,9 +33,10 @@ import struct
 import sys
 import time
 import threading
+import traceback
 import zlib
 
-import websockets
+import aiohttp
 
 from . import utils
 from .activity import BaseActivity
@@ -49,13 +50,19 @@ __all__ = (
     'KeepAliveHandler',
     'VoiceKeepAliveHandler',
     'DiscordVoiceWebSocket',
-    'ResumeWebSocket',
+    'ReconnectWebSocket',
 )
 
-class ResumeWebSocket(Exception):
-    """Signals to initialise via RESUME opcode instead of IDENTIFY."""
-    def __init__(self, shard_id):
+class ReconnectWebSocket(Exception):
+    """Signals to safely reconnect the websocket."""
+    def __init__(self, shard_id, *, resume=True):
         self.shard_id = shard_id
+        self.resume = resume
+        self.op = 'RESUME' if resume else 'IDENTIFY'
+
+class WebSocketClosure(Exception):
+    """An exception to make up for the fact that aiohttp doesn't signal closure."""
+    pass
 
 EventListener = namedtuple('EventListener', 'predicate event result future')
 
@@ -66,12 +73,13 @@ class KeepAliveHandler(threading.Thread):
         shard_id = kwargs.pop('shard_id', None)
         threading.Thread.__init__(self, *args, **kwargs)
         self.ws = ws
+        self._main_thread_id = ws.thread_id
         self.interval = interval
         self.daemon = True
         self.shard_id = shard_id
-        self.msg = 'Keeping websocket alive with sequence %s.'
-        self.block_msg = 'Heartbeat blocked for more than %s seconds.'
-        self.behind_msg = 'Can\'t keep up, websocket is %.1fs behind.'
+        self.msg = 'Keeping shard ID %s websocket alive with sequence %s.'
+        self.block_msg = 'Shard ID %s heartbeat blocked for more than %s seconds.'
+        self.behind_msg = 'Can\'t keep up, shard ID %s websocket is %.1fs behind.'
         self._stop_ev = threading.Event()
         self._last_ack = time.perf_counter()
         self._last_send = time.perf_counter()
@@ -88,13 +96,13 @@ class KeepAliveHandler(threading.Thread):
                 try:
                     f.result()
                 except Exception:
-                    pass
+                    log.exception('An error occurred while stopping the gateway. Ignoring.')
                 finally:
                     self.stop()
                     return
 
             data = self.get_payload()
-            log.debug(self.msg, data['d'])
+            log.debug(self.msg, self.shard_id, data['d'])
             coro = self.ws.send_as_json(data)
             f = asyncio.run_coroutine_threadsafe(coro, loop=self.ws.loop)
             try:
@@ -102,11 +110,18 @@ class KeepAliveHandler(threading.Thread):
                 total = 0
                 while True:
                     try:
-                        f.result(5)
+                        f.result(10)
                         break
                     except concurrent.futures.TimeoutError:
-                        total += 5
-                        log.warning(self.block_msg, total)
+                        total += 10
+                        try:
+                            frame = sys._current_frames()[self._main_thread_id]
+                        except KeyError:
+                            msg = self.block_msg
+                        else:
+                            stack = traceback.format_stack(frame)
+                            msg = '%s\nLoop thread traceback (most recent call last):\n%s' % (self.block_msg, ''.join(stack))
+                        log.warning(msg, self.shard_id, total)
 
             except Exception:
                 self.stop()
@@ -127,14 +142,15 @@ class KeepAliveHandler(threading.Thread):
         self._last_ack = ack_time
         self.latency = ack_time - self._last_send
         if self.latency > 10:
-            log.warning(self.behind_msg, self.latency)
+            log.warning(self.behind_msg, self.shard_id, self.latency)
 
 class VoiceKeepAliveHandler(KeepAliveHandler):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.msg = 'Keeping voice websocket alive with timestamp %s.'
-        self.block_msg = 'Voice heartbeat blocked for more than %s seconds'
-        self.behind_msg = 'Can\'t keep up, voice websocket is %.1fs behind'
+        self.recent_ack_latencies = deque(maxlen=20)
+        self.msg = 'Keeping shard ID %s voice websocket alive with timestamp %s.'
+        self.block_msg = 'Shard ID %s voice heartbeat blocked for more than %s seconds'
+        self.behind_msg = 'High socket latency, shard ID %s heartbeat is %.1fs behind'
 
     def get_payload(self):
         return {
@@ -142,11 +158,18 @@ class VoiceKeepAliveHandler(KeepAliveHandler):
             'd': int(time.time() * 1000)
         }
 
-class DiscordWebSocket(websockets.client.WebSocketClientProtocol):
-    """Implements a WebSocket for Discord's gateway v6.
+    def ack(self):
+        ack_time = time.perf_counter()
+        self._last_ack = ack_time
+        self.latency = ack_time - self._last_send
+        self.recent_ack_latencies.append(self.latency)
 
-    This is created through :func:`create_main_websocket`. Library
-    users should never create this manually.
+class DiscordClientWebSocketResponse(aiohttp.ClientWebSocketResponse):
+    async def close(self, *, code: int = 4000, message: bytes = b'') -> bool:
+        return await super().close(code=code, message=message)
+
+class DiscordWebSocket:
+    """Implements a WebSocket for Discord's gateway v6.
 
     Attributes
     -----------
@@ -199,30 +222,38 @@ class DiscordWebSocket(websockets.client.WebSocketClientProtocol):
     HEARTBEAT_ACK      = 11
     GUILD_SYNC         = 12
 
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.max_size = None
+    def __init__(self, socket, *, loop):
+        self.socket = socket
+        self.loop = loop
+
         # an empty dispatcher to prevent crashes
         self._dispatch = lambda *args: None
         # generic event listeners
         self._dispatch_listeners = []
         # the keep alive
         self._keep_alive = None
+        self.thread_id = threading.get_ident()
 
         # ws related stuff
         self.session_id = None
         self.sequence = None
         self._zlib = zlib.decompressobj()
         self._buffer = bytearray()
+        self._close_code = None
+
+    @property
+    def open(self):
+        return not self.socket.closed
 
     @classmethod
-    async def from_client(cls, client, *, shard_id=None, session=None, sequence=None, resume=False):
+    async def from_client(cls, client, *, initial=False, gateway=None, shard_id=None, session=None, sequence=None, resume=False):
         """Creates a main websocket for Discord from a :class:`Client`.
 
         This is for internal use only.
         """
-        gateway = await client.http.get_gateway()
-        ws = await websockets.connect(gateway, loop=client.loop, klass=cls, compression=None)
+        gateway = gateway or await client.http.get_gateway()
+        socket = await client.http.ws_connect(gateway)
+        ws = cls(socket, loop=client.loop)
 
         # dynamically add attributes needed
         ws.token = client.http.token
@@ -230,6 +261,8 @@ class DiscordWebSocket(websockets.client.WebSocketClientProtocol):
         ws._discord_parsers = client._connection.parsers
         ws._dispatch = client.dispatch
         ws.gateway = gateway
+        ws.call_hooks = client._connection.call_hooks
+        ws._initial_identify = initial
         ws.shard_id = shard_id
         ws.shard_count = client._connection.shard_count
         ws.session_id = session
@@ -238,7 +271,7 @@ class DiscordWebSocket(websockets.client.WebSocketClientProtocol):
 
         client._connection._update_references(ws)
 
-        log.info('Created websocket connected to %s', gateway)
+        log.debug('Created websocket connected to %s', gateway)
 
         # poll event for OP Hello
         await ws.poll_event()
@@ -248,14 +281,7 @@ class DiscordWebSocket(websockets.client.WebSocketClientProtocol):
             return ws
 
         await ws.resume()
-        try:
-            await ws.ensure_open()
-        except websockets.exceptions.ConnectionClosed:
-            # ws got closed so let's just do a regular IDENTIFY connect.
-            log.info('RESUME failed (the websocket decided to close) for Shard ID %s. Retrying.', shard_id)
-            return await cls.from_client(client, shard_id=shard_id)
-        else:
-            return ws
+        return ws
 
     def wait_for(self, event, predicate, result=None):
         """Waits for a DISPATCH'd event that meets the predicate.
@@ -317,6 +343,7 @@ class DiscordWebSocket(websockets.client.WebSocketClientProtocol):
                 'afk': False
             }
 
+        await self.call_hooks('before_identify', self.shard_id, initial=self._initial_identify)
         await self.send_as_json(payload)
         log.info('Shard ID %s has sent the IDENTIFY payload.', self.shard_id)
 
@@ -366,9 +393,9 @@ class DiscordWebSocket(websockets.client.WebSocketClientProtocol):
                 # "reconnect" can only be handled by the Client
                 # so we terminate our connection and raise an
                 # internal exception signalling to reconnect.
-                log.info('Received RECONNECT opcode.')
+                log.debug('Received RECONNECT opcode.')
                 await self.close()
-                raise ResumeWebSocket(self.shard_id)
+                raise ReconnectWebSocket(self.shard_id)
 
             if op == self.HEARTBEAT_ACK:
                 self._keep_alive.ack()
@@ -389,16 +416,14 @@ class DiscordWebSocket(websockets.client.WebSocketClientProtocol):
 
             if op == self.INVALIDATE_SESSION:
                 if data is True:
-                    await asyncio.sleep(5.0)
                     await self.close()
-                    raise ResumeWebSocket(self.shard_id)
+                    raise ReconnectWebSocket(self.shard_id)
 
                 self.sequence = None
                 self.session_id = None
                 log.info('Shard ID %s session has been invalidated.', self.shard_id)
-                await asyncio.sleep(5.0)
-                await self.identify()
-                return
+                await self.close(code=1000)
+                raise ReconnectWebSocket(self.shard_id, resume=False)
 
             log.warning('Unknown OP code %s.', op)
             return
@@ -409,11 +434,15 @@ class DiscordWebSocket(websockets.client.WebSocketClientProtocol):
             self._trace = trace = data.get('_trace', [])
             self.sequence = msg['s']
             self.session_id = data['session_id']
+            # pass back shard ID to ready handler
+            data['__shard_id__'] = self.shard_id
             log.info('Shard ID %s has connected to Gateway: %s (Session ID: %s).',
                      self.shard_id, ', '.join(trace), self.session_id)
 
         elif event == 'RESUMED':
             self._trace = trace = data.get('_trace', [])
+            # pass back the shard ID to the resumed handler
+            data['__shard_id__'] = self.shard_id
             log.info('Shard ID %s has successfully RESUMED session %s under trace %s.',
                      self.shard_id, self.session_id, ', '.join(trace))
 
@@ -455,7 +484,8 @@ class DiscordWebSocket(websockets.client.WebSocketClientProtocol):
         heartbeat = self._keep_alive
         return float('inf') if heartbeat is None else heartbeat.latency
 
-    def _can_handle_close(self, code):
+    def _can_handle_close(self):
+        code = self._close_code or self.socket.close_code
         return code not in (1000, 4004, 4010, 4011)
 
     async def poll_event(self):
@@ -467,26 +497,45 @@ class DiscordWebSocket(websockets.client.WebSocketClientProtocol):
             The websocket connection was terminated for unhandled reasons.
         """
         try:
-            msg = await self.recv()
-            await self.received_message(msg)
-        except websockets.exceptions.ConnectionClosed as exc:
-            if self._can_handle_close(exc.code):
-                log.info('Websocket closed with %s (%s), attempting a reconnect.', exc.code, exc.reason)
-                raise ResumeWebSocket(self.shard_id) from exc
-            else:
-                log.info('Websocket closed with %s (%s), cannot reconnect.', exc.code, exc.reason)
-                raise ConnectionClosed(exc, shard_id=self.shard_id) from exc
+            msg = await self.socket.receive(timeout=self._max_heartbeat_timeout)
+            if msg.type is aiohttp.WSMsgType.TEXT:
+                await self.received_message(msg.data)
+            elif msg.type is aiohttp.WSMsgType.BINARY:
+                await self.received_message(msg.data)
+            elif msg.type is aiohttp.WSMsgType.ERROR:
+                log.debug('Received %s', msg)
+                raise msg.data
+            elif msg.type in (aiohttp.WSMsgType.CLOSED, aiohttp.WSMsgType.CLOSING, aiohttp.WSMsgType.CLOSE):
+                log.debug('Received %s', msg)
+                raise WebSocketClosure
+        except (asyncio.TimeoutError, WebSocketClosure) as e:
+            # Ensure the keep alive handler is closed
+            if self._keep_alive:
+                self._keep_alive.stop()
+                self._keep_alive = None
+
+            if isinstance(e, asyncio.TimeoutError):
+                log.info('Timed out receiving packet. Attempting a reconnect.')
+                raise ReconnectWebSocket(self.shard_id) from None
+
+            code = self._close_code or self.socket.close_code
+            if self._can_handle_close():
+                log.info('Websocket closed with %s, attempting a reconnect.', code)
+                raise ReconnectWebSocket(self.shard_id) from None
+            elif self.socket.close_code is not None:
+                log.info('Websocket closed with %s, cannot reconnect.', code)
+                raise ConnectionClosed(self.socket, shard_id=self.shard_id, code=code) from None
 
     async def send(self, data):
         self._dispatch('socket_raw_send', data)
-        await super().send(data)
+        await self.socket.send_str(data)
 
     async def send_as_json(self, data):
         try:
             await self.send(utils.to_json(data))
-        except websockets.exceptions.ConnectionClosed as exc:
-            if not self._can_handle_close(exc.code):
-                raise ConnectionClosed(exc, shard_id=self.shard_id) from exc
+        except RuntimeError as exc:
+            if not self._can_handle_close():
+                raise ConnectionClosed(self.socket, shard_id=self.shard_id) from exc
 
     async def change_presence(self, *, activity=None, status=None, afk=False, since=0.0):
         if activity is not None:
@@ -518,15 +567,25 @@ class DiscordWebSocket(websockets.client.WebSocketClientProtocol):
         }
         await self.send_as_json(payload)
 
-    async def request_chunks(self, guild_id, query, limit):
+    async def request_chunks(self, guild_id, query=None, *, limit, user_ids=None, nonce=None):
         payload = {
             'op': self.REQUEST_MEMBERS,
             'd': {
-                'guild_id': str(guild_id),
-                'query': query,
+                'guild_id': guild_id,
                 'limit': limit
             }
         }
+
+        if nonce:
+            payload['d']['nonce'] = nonce
+
+        if user_ids:
+            payload['d']['user_ids'] = user_ids
+
+        if query is not None:
+            payload['d']['query'] = query
+
+
         await self.send_as_json(payload)
 
     async def voice_state(self, guild_id, channel_id, self_mute=False, self_deaf=False):
@@ -543,19 +602,15 @@ class DiscordWebSocket(websockets.client.WebSocketClientProtocol):
         log.debug('Updating our voice state to %s.', payload)
         await self.send_as_json(payload)
 
-    async def close(self, code=4000, reason=''):
+    async def close(self, code=4000):
         if self._keep_alive:
             self._keep_alive.stop()
+            self._keep_alive = None
 
-        await super().close(code, reason)
+        self._close_code = code
+        await self.socket.close(code=code)
 
-    async def close_connection(self, *args, **kwargs):
-        if self._keep_alive:
-            self._keep_alive.stop()
-
-        await super().close_connection(*args, **kwargs)
-
-class DiscordVoiceWebSocket(websockets.client.WebSocketClientProtocol):
+class DiscordVoiceWebSocket:
     """Implements the websocket protocol for handling voice connections.
 
     Attributes
@@ -599,14 +654,15 @@ class DiscordVoiceWebSocket(websockets.client.WebSocketClientProtocol):
     CLIENT_CONNECT      = 12
     CLIENT_DISCONNECT   = 13
 
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.max_size = None
+    def __init__(self, socket, loop):
+        self.ws = socket
+        self.loop = loop
         self._keep_alive = None
+        self._close_code = None
 
     async def send_as_json(self, data):
         log.debug('Sending voice websocket frame: %s.', data)
-        await self.send(utils.to_json(data))
+        await self.ws.send_str(utils.to_json(data))
 
     async def resume(self):
         state = self._connection
@@ -637,10 +693,13 @@ class DiscordVoiceWebSocket(websockets.client.WebSocketClientProtocol):
     async def from_client(cls, client, *, resume=False):
         """Creates a voice websocket for the :class:`VoiceClient`."""
         gateway = 'wss://' + client.endpoint + '/?v=4'
-        ws = await websockets.connect(gateway, loop=client.loop, klass=cls, compression=None)
+        http = client._state.http
+        socket = await http.ws_connect(gateway, compress=15)
+        ws = cls(socket, loop=client.loop)
         ws.gateway = gateway
         ws._connection = client
         ws._max_heartbeat_timeout = 60.0
+        ws.thread_id = threading.get_ident()
 
         if resume:
             await ws.resume()
@@ -702,7 +761,7 @@ class DiscordVoiceWebSocket(websockets.client.WebSocketClientProtocol):
             await self.load_secret_key(data)
         elif op == self.HELLO:
             interval = data['heartbeat_interval'] / 1000.0
-            self._keep_alive = VoiceKeepAliveHandler(ws=self, interval=interval)
+            self._keep_alive = VoiceKeepAliveHandler(ws=self, interval=min(interval, 5.0))
             self._keep_alive.start()
 
     async def initial_connection(self, data):
@@ -735,6 +794,21 @@ class DiscordVoiceWebSocket(websockets.client.WebSocketClientProtocol):
 
         await self.client_connect()
 
+    @property
+    def latency(self):
+        """:class:`float`: Latency between a HEARTBEAT and its HEARTBEAT_ACK in seconds."""
+        heartbeat = self._keep_alive
+        return float('inf') if heartbeat is None else heartbeat.latency
+
+    @property
+    def average_latency(self):
+        """:class:`list`: Average of last 20 HEARTBEAT latencies."""
+        heartbeat = self._keep_alive
+        if heartbeat is None:
+            return float('inf')
+
+        return sum(heartbeat.recent_ack_latencies) / len(heartbeat.recent_ack_latencies)
+
     async def load_secret_key(self, data):
         log.info('received secret key for voice connection')
         self._connection.secret_key = data.get('secret_key')
@@ -742,14 +816,20 @@ class DiscordVoiceWebSocket(websockets.client.WebSocketClientProtocol):
         await self.speak(False)
 
     async def poll_event(self):
-        try:
-            msg = await asyncio.wait_for(self.recv(), timeout=30.0)
-            await self.received_message(json.loads(msg))
-        except websockets.exceptions.ConnectionClosed as exc:
-            raise ConnectionClosed(exc, shard_id=None) from exc
+        # This exception is handled up the chain
+        msg = await asyncio.wait_for(self.ws.receive(), timeout=30.0)
+        if msg.type is aiohttp.WSMsgType.TEXT:
+            await self.received_message(json.loads(msg.data))
+        elif msg.type is aiohttp.WSMsgType.ERROR:
+            log.debug('Received %s', msg)
+            raise ConnectionClosed(self.ws, shard_id=None) from msg.data
+        elif msg.type in (aiohttp.WSMsgType.CLOSED, aiohttp.WSMsgType.CLOSE, aiohttp.WSMsgType.CLOSING):
+            log.debug('Received %s', msg)
+            raise ConnectionClosed(self.ws, shard_id=None, code=self._close_code)
 
-    async def close_connection(self, *args, **kwargs):
-        if self._keep_alive:
+    async def close(self, code=1000):
+        if self._keep_alive is not None:
             self._keep_alive.stop()
 
-        await super().close_connection(*args, **kwargs)
+        self._close_code = code
+        await self.ws.close(code=code)
