@@ -27,6 +27,7 @@ DEALINGS IN THE SOFTWARE.
 from collections import namedtuple
 
 import discord.abc
+from .flags import PublicUserFlags
 from .utils import snowflake_time, _bytes_to_base64_data, parse_time
 from .enums import DefaultAvatar, RelationshipType, UserFlags, HypeSquadHouse, PremiumType, try_enum
 from .errors import ClientException
@@ -82,7 +83,7 @@ class Profile(namedtuple('Profile', 'flags user mutual_guilds connected_accounts
 _BaseUser = discord.abc.User
 
 class BaseUser(_BaseUser):
-    __slots__ = ('name', 'id', 'discriminator', 'avatar', 'bot', 'system', '_state')
+    __slots__ = ('name', 'id', 'discriminator', 'avatar', 'bot', 'system', '_public_flags', '_state')
 
     def __init__(self, *, state, data):
         self._state = state
@@ -105,6 +106,7 @@ class BaseUser(_BaseUser):
         self.id = int(data['id'])
         self.discriminator = data['discriminator']
         self.avatar = data['avatar']
+        self._public_flags = data.get('public_flags', 0)
         self.bot = data.get('bot', False)
         self.system = data.get('system', False)
 
@@ -130,9 +132,14 @@ class BaseUser(_BaseUser):
             'bot': self.bot,
         }
 
+    @property
+    def public_flags(self):
+        """:class:`PublicUserFlags`: The publicly available flags the user has."""
+        return PublicUserFlags._from_value(self._public_flags)
+
     @property
     def avatar_url(self):
-        """Returns an :class:`Asset` for the avatar the user has.
+        """:class:`Asset`: Returns an :class:`Asset` for the avatar the user has.
 
         If the user does not have a traditional avatar, an asset for
         the default avatar is returned instead.
@@ -197,7 +204,7 @@ class BaseUser(_BaseUser):
         """:class:`Colour`: A property that returns a colour denoting the rendered colour
         for the user. This always returns :meth:`Colour.default`.
 
-        There is an alias for this named :meth:`color`.
+        There is an alias for this named :attr:`color`.
         """
         return Colour.default()
 
@@ -206,7 +213,7 @@ class BaseUser(_BaseUser):
         """:class:`Colour`: A property that returns a color denoting the rendered color
         for the user. This always returns :meth:`Colour.default`.
 
-        There is an alias for this named :meth:`colour`.
+        There is an alias for this named :attr:`colour`.
         """
         return self.colour
 
@@ -296,7 +303,7 @@ class ClientUser(BaseUser):
     discriminator: :class:`str`
         The user's discriminator. This is given when the username has conflicts.
     avatar: Optional[:class:`str`]
-        The avatar hash the user has. Could be None.
+        The avatar hash the user has. Could be ``None``.
     bot: :class:`bool`
         Specifies if the user is a bot account.
     system: :class:`bool`
@@ -710,7 +717,7 @@ class User(BaseUser, discord.abc.Messageable):
 
     @property
     def relationship(self):
-        """Returns the :class:`Relationship` with this user if applicable, ``None`` otherwise.
+        """Optional[:class:`Relationship`]: Returns the :class:`Relationship` with this user if applicable, ``None`` otherwise.
 
         .. note::
 
