@@ -28,6 +28,7 @@ import asyncio
 import json
 import time
 import re
+from urllib.parse import quote as _uriquote
 
 import aiohttp
 
@@ -36,6 +37,7 @@ from .errors import InvalidArgument, HTTPException, Forbidden, NotFound
 from .enums import try_enum, WebhookType
 from .user import BaseUser, User
 from .asset import Asset
+from .mixins import Hashable
 
 __all__ = (
     'WebhookAdapter',
@@ -53,7 +55,7 @@ class WebhookAdapter:
         The webhook that owns this adapter.
     """
 
-    BASE = 'https://discordapp.com/api/v7'
+    BASE = 'https://discord.com/api/v7'
 
     def _prepare(self, webhook):
         self._webhook_id = webhook.id
@@ -83,11 +85,11 @@ class WebhookAdapter:
         """
         raise NotImplementedError()
 
-    def delete_webhook(self):
-        return self.request('DELETE', self._request_url)
+    def delete_webhook(self, *, reason=None):
+        return self.request('DELETE', self._request_url, reason=reason)
 
-    def edit_webhook(self, **payload):
-        return self.request('PATCH', self._request_url, payload=payload)
+    def edit_webhook(self, *, reason=None, **payload):
+        return self.request('PATCH', self._request_url, payload=payload, reason=reason)
 
     def handle_execution_response(self, data, *, wait):
         """Transforms the webhook execution response into something
@@ -173,13 +175,16 @@ class AsyncWebhookAdapter(WebhookAdapter):
         self.session = session
         self.loop = asyncio.get_event_loop()
 
-    async def request(self, verb, url, payload=None, multipart=None, *, files=None):
+    async def request(self, verb, url, payload=None, multipart=None, *, files=None, reason=None):
         headers = {}
         data = None
         files = files or []
         if payload:
             headers['Content-Type'] = 'application/json'
             data = utils.to_json(payload)
+        
+        if reason:
+            headers['X-Audit-Log-Reason'] = _uriquote(reason, safe='/ ')
 
         if multipart:
             data = aiohttp.FormData()
@@ -259,13 +264,16 @@ class RequestsWebhookAdapter(WebhookAdapter):
         self.session = session or requests
         self.sleep = sleep
 
-    def request(self, verb, url, payload=None, multipart=None, *, files=None):
+    def request(self, verb, url, payload=None, multipart=None, *, files=None, reason=None):
         headers = {}
         data = None
         files = files or []
         if payload:
             headers['Content-Type'] = 'application/json'
             data = utils.to_json(payload)
+    
+        if reason:
+            headers['X-Audit-Log-Reason'] = _uriquote(reason, safe='/ ')
 
         if multipart is not None:
             data = {'payload_json': multipart.pop('payload_json')}
@@ -359,7 +367,7 @@ class _PartialWebhookState:
     def __getattr__(self, attr):
         raise AttributeError('PartialWebhookState does not support {0!r}.'.format(attr))
 
-class Webhook:
+class Webhook(Hashable):
     """Represents a Discord webhook.
 
     Webhooks are a form to send messages to channels in Discord without a
@@ -400,6 +408,23 @@ class Webhook:
         webhook = Webhook.partial(123456, 'abcdefg', adapter=RequestsWebhookAdapter())
         webhook.send('Hello World', username='Foo')
 
+    .. container:: operations
+    
+        .. describe:: x == y
+        
+            Checks if two webhooks are equal.
+            
+        .. describe:: x != y
+        
+            Checks if two webhooks are not equal.
+            
+        .. describe:: hash(x)
+        
+            Returns the webhooks's hash.
+            
+    .. versionchanged:: 1.4
+        Webhooks are now comparable and hashable.
+    
     Attributes
     ------------
     id: :class:`int`
@@ -453,15 +478,13 @@ class Webhook:
 
     @property
     def url(self):
-        """Returns the webhook's url."""
-        return 'https://discordapp.com/api/webhooks/{}/{}'.format(self.id, self.token)
+        """:class:`str` : Returns the webhook's url."""
+        return 'https://discord.com/api/webhooks/{}/{}'.format(self.id, self.token)
 
     @classmethod
     def partial(cls, id, token, *, adapter):
         """Creates a partial :class:`Webhook`.
 
-        A partial webhook is just a webhook object with an ID and a token.
-
         Parameters
         -----------
         id: :class:`int`
@@ -472,6 +495,12 @@ class Webhook:
             The webhook adapter to use when sending requests. This is
             typically :class:`AsyncWebhookAdapter` for :doc:`aiohttp <aio:index>` or
             :class:`RequestsWebhookAdapter` for :doc:`req:index`.
+
+        Returns
+        --------
+        :class:`Webhook`
+            A partial :class:`Webhook`.
+            A partial webhook is just a webhook object with an ID and a token.
         """
 
         if not isinstance(adapter, WebhookAdapter):
@@ -502,9 +531,15 @@ class Webhook:
         -------
         InvalidArgument
             The URL is invalid.
+
+        Returns
+        --------
+        :class:`Webhook`
+            A partial :class:`Webhook`.
+            A partial webhook is just a webhook object with an ID and a token.
         """
 
-        m = re.search(r'discordapp.com/api/webhooks/(?P<id>[0-9]{17,21})/(?P<token>[A-Za-z0-9\.\-\_]{60,68})', url)
+        m = re.search(r'discord(?:app)?.com/api/webhooks/(?P<id>[0-9]{17,21})/(?P<token>[A-Za-z0-9\.\-\_]{60,68})', url)
         if m is None:
             raise InvalidArgument('Invalid webhook URL given.')
         data = m.groupdict()
@@ -560,7 +595,7 @@ class Webhook:
 
     @property
     def avatar_url(self):
-        """Returns an :class:`Asset` for the avatar the webhook has.
+        """:class:`Asset`: Returns an :class:`Asset` for the avatar the webhook has.
 
         If the webhook does not have a traditional avatar, an asset for
         the default avatar is returned instead.
@@ -612,7 +647,7 @@ class Webhook:
         url = '/avatars/{0.id}/{0.avatar}.{1}?size={2}'.format(self, format, size)
         return Asset(self._state, url)
 
-    def delete(self):
+    def delete(self, *, reason=None):
         """|maybecoro|
 
         Deletes this Webhook.
@@ -620,6 +655,13 @@ class Webhook:
         If the webhook is constructed with a :class:`RequestsWebhookAdapter` then this is
         not a coroutine.
 
+        Parameters
+        ------------
+        reason: Optional[:class:`str`]
+            The reason for deleting this webhook. Shows up on the audit log.
+
+            .. versionadded:: 1.4
+
         Raises
         -------
         HTTPException
@@ -634,9 +676,9 @@ class Webhook:
         if self.token is None:
             raise InvalidArgument('This webhook does not have a token associated with it')
 
-        return self._adapter.delete_webhook()
+        return self._adapter.delete_webhook(reason=reason)
 
-    def edit(self, **kwargs):
+    def edit(self, *, reason=None, **kwargs):
         """|maybecoro|
 
         Edits this Webhook.
@@ -645,11 +687,15 @@ class Webhook:
         not a coroutine.
 
         Parameters
-        -------------
+        ------------
         name: Optional[:class:`str`]
             The webhook's new default name.
         avatar: Optional[:class:`bytes`]
             A :term:`py:bytes-like object` representing the webhook's new default avatar.
+        reason: Optional[:class:`str`]
+            The reason for editing this webhook. Shows up on the audit log.
+
+            .. versionadded:: 1.4
 
         Raises
         -------
@@ -685,10 +731,10 @@ class Webhook:
             else:
                 payload['avatar'] = None
 
-        return self._adapter.edit_webhook(**payload)
+        return self._adapter.edit_webhook(reason=reason, **payload)
 
     def send(self, content=None, *, wait=False, username=None, avatar_url=None, tts=False,
-                                    file=None, files=None, embed=None, embeds=None):
+                                    file=None, files=None, embed=None, embeds=None, allowed_mentions=None):
         """|maybecoro|
 
         Sends a message using the webhook.
@@ -732,6 +778,10 @@ class Webhook:
         embeds: List[:class:`Embed`]
             A list of embeds to send with the content. Maximum of 10. This cannot
             be mixed with the ``embed`` parameter.
+        allowed_mentions: :class:`AllowedMentions`
+            Controls the mentions being processed in this message.
+
+            .. versionadded:: 1.4
 
         Raises
         --------
@@ -777,6 +827,16 @@ class Webhook:
         if username:
             payload['username'] = username
 
+        previous_mentions = getattr(self._state, 'allowed_mentions', None)
+
+        if allowed_mentions:
+            if previous_mentions is not None:
+                payload['allowed_mentions'] = previous_mentions.merge(allowed_mentions).to_dict()
+            else:
+                payload['allowed_mentions'] = allowed_mentions.to_dict()
+        elif previous_mentions is not None:
+            payload['allowed_mentions'] = previous_mentions.to_dict()
+
         return self._adapter.execute_webhook(wait=wait, file=file, files=files, payload=payload)
 
     def execute(self, *args, **kwargs):
